@@ -1,5 +1,5 @@
-/*
- * conn.c
+/**
+ * Low-level connection handling.
  *
  * Does all this gloriously nifty connection handling stuff...
  *
@@ -19,10 +19,10 @@
 #include "win32dep.h"
 #endif
 
-/*
+/**
  * In OSCAR, every connection has a set of SNAC groups associated
  * with it.  These are the groups that you can send over this connection
- * without being guarenteed a "Not supported" SNAC error.  
+ * without being guaranteed a "Not supported" SNAC error.  
  *
  * The grand theory of things says that these associations transcend 
  * what libfaim calls "connection types" (conn->type).  You can probably
@@ -43,7 +43,7 @@
  * easy and deliver this SNAC for you, but there isn't one there.
  *
  * Here comes the good bit.  Without even letting anyone know, particularly
- * the module that decided to send this SNAC, and definitly not that twit
+ * the module that decided to send this SNAC, and definitely not that twit
  * in Greenland, you send out a service request.  In this request, you have
  * marked the need for a connection supporting group 0x000e.  A few seconds
  * later, you receive a service redirect with an IP address and a cookie in
@@ -191,11 +191,9 @@ static void connkill_real(aim_session_t *sess, aim_conn_t **deadconn)
 }
 
 /**
- * aim_connrst - Clears out connection list, killing remaining connections.
- * @sess: Session to be cleared
- *
- * Clears out the connection list and kills any connections left.
+ * Clears out the connection list, killing remaining connections.
  *
+ * @param sess Session to be cleared.
  */
 static void aim_connrst(aim_session_t *sess)
 {
@@ -217,11 +215,9 @@ static void aim_connrst(aim_session_t *sess)
 }
 
 /**
- * aim_conn_init - Reset a connection to default values.
- * @deadconn: Connection to be reset
- *
- * Initializes and/or resets a connection structure.
+ * Initializes and/or resets a connection structure to the default values.
  *
+ * @param deadconn Connection to be reset.
  */
 static void aim_conn_init(aim_conn_t *deadconn)
 {
@@ -243,11 +239,10 @@ static void aim_conn_init(aim_conn_t *deadconn)
 }
 
 /**
- * aim_conn_getnext - Gets a new connection structure.
- * @sess: Session
- *
  * Allocate a new empty connection structure.
  *
+ * @param sess Session
+ * @return Returns the new connection structure.
  */
 static aim_conn_t *aim_conn_getnext(aim_session_t *sess)
 {
@@ -272,13 +267,11 @@ static aim_conn_t *aim_conn_getnext(aim_session_t *sess)
 }
 
 /**
- * aim_conn_kill - Close and free a connection.
- * @sess: Session for the connection
- * @deadconn: Connection to be freed
- *
  * Close, clear, and free a connection structure. Should never be
  * called from within libfaim.
  *
+ * @param sess Session for the connection.
+ * @param deadconn Connection to be freed.
  */
 faim_export void aim_conn_kill(aim_session_t *sess, aim_conn_t **deadconn)
 {
@@ -304,15 +297,16 @@ faim_export void aim_conn_kill(aim_session_t *sess, aim_conn_t **deadconn)
 }
 
 /**
- * aim_conn_close - Close a connection
- * @deadconn: Connection to close
- *
  * Close (but not free) a connection.
  *
  * This leaves everything untouched except for clearing the 
  * handler list and setting the fd to -1 (used to recognize
  * dead connections).  It will also remove cookies if necessary.
  *
+ * Why only if fd >= 3?  Seems rather implementation specific...
+ * fd's do not have to be distributed in a particular order, do they?
+ *
+ * @param deadconn The connection to close.
  */
 faim_export void aim_conn_close(aim_conn_t *deadconn)
 {
@@ -333,16 +327,16 @@ faim_export void aim_conn_close(aim_conn_t *deadconn)
 }
 
 /**
- * aim_getconn_type - Find a connection of a specific type
- * @sess: Session to search
- * @type: Type of connection to look for
+ * Locates a connection of the specified type in the 
+ * specified session.
  *
- * Searches for a connection of the specified type in the 
- * specified session.  Returns the first connection of that
- * type found.
+ * XXX - Except for rendezvous, all uses of this should be removed and
+ * aim_conn_findbygroup() should be used instead.
  *
- * XXX except for RENDEZVOUS, all uses of this should be removed and
- * use aim_conn_findbygroup() instead.
+ * @param sess The session to search.
+ * @param type The type of connection to look for.
+ * @return Returns the first connection found of the given target type,
+ *         or NULL if none could be found.
  */
 faim_export aim_conn_t *aim_getconn_type(aim_session_t *sess, int type)
 {
@@ -383,18 +377,21 @@ faim_export aim_conn_t *aim_getconn_fd(aim_session_t *sess, int fd)
 }
 
 /**
- * aim_proxyconnect - An extrememly quick and dirty SOCKS5 interface. 
- * @sess: Session to connect
- * @host: Host to connect to
- * @port: Port to connect to
- * @statusret: Return value of the connection
+ * Handle normal connections or SOCKS5 via an extrememly quick and 
+ * dirty SOCKS5 interface. 
  *
  * Attempts to connect to the specified host via the configured
  * proxy settings, if present.  If no proxy is configured for
  * this session, the connection is done directly.
  *
- * XXX this is really awful.
+ * XXX - this is really awful.
+ * XXX - Split the SOCKS5 and the normal connection stuff into two
+ *        separate functions.
  *
+ * @param sess Session to connect.
+ * @param host Host to connect to.
+ * @param port Port to connect to.
+ * @param statusret Return value of the connection.
  */
 static int aim_proxyconnect(aim_session_t *sess, const char *host, fu16_t port, fu32_t *statusret)
 {
@@ -532,11 +529,11 @@ static int aim_proxyconnect(aim_session_t *sess, const char *host, fu16_t port,
 
 		fd = socket(hp->h_addrtype, SOCK_STREAM, 0);
 
-		if (sess->flags & AIM_SESS_FLAGS_NONBLOCKCONNECT)
+		if (sess->nonblocking)
 			fcntl(fd, F_SETFL, O_NONBLOCK); /* XXX save flags */
 
 		if (connect(fd, (struct sockaddr *)&sa, sizeof(struct sockaddr_in)) < 0) {
-			if (sess->flags & AIM_SESS_FLAGS_NONBLOCKCONNECT) {
+			if (sess->nonblocking) {
 				if ((errno == EINPROGRESS) || (errno == EINTR)) {
 					if (statusret)
 						*statusret |= AIM_CONN_STATUS_INPROGRESS;
@@ -551,17 +548,16 @@ static int aim_proxyconnect(aim_session_t *sess, const char *host, fu16_t port,
 }
 
 /**
- * aim_cloneconn - clone an aim_conn_t
- * @sess: session containing parent
- * @src: connection to clone
+ * Clone an aim_conn_t.
  *
  * A new connection is allocated, and the values are filled in
  * appropriately. Note that this function sets the new connnection's
  * ->priv pointer to be equal to that of its parent: only the pointer
  * is copied, not the data it points to.
  *
- * This function returns a pointer to the new aim_conn_t, or %NULL on
- * error
+ * @param sess The session containing this connection.
+ * @param src The connection to clone.
+ * @return Returns a pointer to the new aim_conn_t, or %NULL on error.
  */
 faim_internal aim_conn_t *aim_cloneconn(aim_session_t *sess, aim_conn_t *src)
 {
@@ -595,11 +591,6 @@ faim_internal aim_conn_t *aim_cloneconn(aim_session_t *sess, aim_conn_t *src)
 }
 
 /**
- * aim_newconn - Open a new connection
- * @sess: Session to create connection in
- * @type: Type of connection to create
- * @dest: Host to connect to (in "host:port" syntax)
- *
  * Opens a new connection to the specified dest host of specified
  * type, using the proxy settings if available.  If @host is %NULL,
  * the connection is allocated and returned, but no connection 
@@ -607,6 +598,9 @@ faim_internal aim_conn_t *aim_cloneconn(aim_session_t *sess, aim_conn_t *src)
  *
  * FIXME: Return errors in a more sane way.
  *
+ * @param sess Session to create connection in
+ * @param type Type of connection to create
+ * @param dest Host to connect to (in "host:port" syntax)
  */
 faim_export aim_conn_t *aim_newconn(aim_session_t *sess, int type, const char *dest)
 {
@@ -661,34 +655,11 @@ faim_export aim_conn_t *aim_newconn(aim_session_t *sess, int type, const char *d
 }
 
 /**
- * aim_conngetmaxfd - Return the highest valued file discriptor in session
- * @sess: Session to search
- *
- * Returns the highest valued filed descriptor of all open 
- * connections in @sess.
- *
- */
-faim_export int aim_conngetmaxfd(aim_session_t *sess)
-{
-	int j;
-	aim_conn_t *cur;
-
-	for (cur = sess->connlist, j = 0; cur; cur = cur->next) {
-		if (cur->fd > j)
-			j = cur->fd;
-	}
-
-	return j;
-}
-
-/**
- * aim_conn_in_sess - Predicate to test the precense of a connection in a sess
- * @sess: Session to look in
- * @conn: Connection to look for
- *
- * Searches @sess for the passed connection.  Returns 1 if its present,
- * zero otherwise.
+ * Searches @sess for the passed connection.
  *
+ * @param sess Session in which to look.
+ * @param conn Connection to look for.
+ * @return Returns 1 if the passed connection is present, zero otherwise.
  */
 faim_export int aim_conn_in_sess(aim_session_t *sess, aim_conn_t *conn)
 {
@@ -703,11 +674,6 @@ faim_export int aim_conn_in_sess(aim_session_t *sess, aim_conn_t *conn)
 }
 
 /**
- * aim_select - Wait for a socket with data or timeout
- * @sess: Session to wait on
- * @timeout: How long to wait
- * @status: Return status
- *
  * Waits for a socket with data or for timeout, whichever comes first.
  * See select(2).
  * 
@@ -717,6 +683,10 @@ faim_export int aim_conn_in_sess(aim_session_t *sess, aim_conn_t *conn)
  *    1  outgoing data pending (%NULL returned)
  *    2  incoming data pending (connection with pending data returned)
  *
+ * @param sess Session to wait on
+ * @param timeout How long to wait
+ * @param status Return status
+ * @return If @status is 2, returns connection with pending data, otherwise %NULL
  */ 
 faim_export aim_conn_t *aim_select(aim_session_t *sess, struct timeval *timeout, int *status)
 {
@@ -785,11 +755,8 @@ faim_export aim_conn_t *aim_select(aim_session_t *sess, struct timeval *timeout,
 }
 
 /**
- * aim_conn_setlatency - Set a forced latency value for connection
- * @conn: Conn to set latency for
- * @newval: Number of seconds to force between transmits
- *
- * Causes @newval seconds to be spent between transmits on a connection.
+ * Set a forced latency value for connection.  Basically causes 
+ * @newval seconds to be spent between transmits on a connection.
  *
  * This is my lame attempt at overcoming not understanding the rate
  * limiting. 
@@ -797,6 +764,9 @@ faim_export aim_conn_t *aim_select(aim_session_t *sess, struct timeval *timeout,
  * XXX: This should really be replaced with something that scales and
  * backs off like the real rate limiting does.
  *
+ * @param conn Conn to set latency for.
+ * @param newval Number of seconds to force between transmits.
+ * @return Returns -1 if the connection does not exist, zero otherwise.
  */
 faim_export int aim_conn_setlatency(aim_conn_t *conn, int newval)
 {
@@ -811,11 +781,7 @@ faim_export int aim_conn_setlatency(aim_conn_t *conn, int newval)
 }
 
 /**
- * aim_setupproxy - Configure a proxy for this session
- * @sess: Session to set proxy for
- * @server: SOCKS server
- * @username: SOCKS username
- * @password: SOCKS password
+ * Configure a proxy for this session.
  *
  * Call this with your SOCKS5 proxy server parameters before
  * the first call to aim_newconn().  If called with all %NULL
@@ -823,6 +789,10 @@ faim_export int aim_conn_setlatency(aim_conn_t *conn, int newval)
  *
  * Set username and password to %NULL if not applicable.
  *
+ * @param sess Session to set proxy for.
+ * @param server SOCKS server.
+ * @param username SOCKS username.
+ * @param password SOCKS password.
  */
 faim_export void aim_setupproxy(aim_session_t *sess, const char *server, const char *username, const char *password)
 {
@@ -852,15 +822,14 @@ static void defaultdebugcb(aim_session_t *sess, int level, const char *format, v
 }
 
 /**
- * aim_session_init - Initializes a session structure
- * @sess: Session to initialize
- * @flags: Flags to use. Any of %AIM_SESS_FLAGS %OR'd together.
- * @debuglevel: Level of debugging output (zero is least)
- *
- * Sets up the initial values for a session.
+ * Initializes a session structure by setting the initial values 
+ * stuff in the aim_session_t struct.
  *
+ * @param sess Session to initialize.
+ * @param nonblocking Set to true if you want connections to be non-blocking.
+ * @param debuglevel Level of debugging output (zero is least).
  */
-faim_export void aim_session_init(aim_session_t *sess, fu32_t flags, int debuglevel)
+faim_export void aim_session_init(aim_session_t *sess, int nonblocking, int debuglevel)
 {
 
 	if (!sess)
@@ -872,16 +841,16 @@ faim_export void aim_session_init(aim_session_t *sess, fu32_t flags, int debugle
 	sess->queue_incoming = NULL;
 	aim_initsnachash(sess);
 	sess->msgcookies = NULL;
-	sess->icq_info = NULL;
-	sess->oft_info = NULL;
-	sess->snacid_next = 0x00000001;
-
-	sess->flags = 0;
+	sess->nonblocking = nonblocking;
 	sess->debug = debuglevel;
 	sess->debugcb = defaultdebugcb;
-
 	sess->modlistv = NULL;
+	sess->snacid_next = 0x00000001;
 
+	sess->locate.userinfo = NULL;
+	sess->locate.torequest = NULL;
+	sess->locate.requested = NULL;
+	sess->locate.waiting_for_response = FALSE;
 	sess->ssi.received_data = 0;
 	sess->ssi.numitems = 0;
 	sess->ssi.official = NULL;
@@ -889,16 +858,10 @@ faim_export void aim_session_init(aim_session_t *sess, fu32_t flags, int debugle
 	sess->ssi.pending = NULL;
 	sess->ssi.timestamp = (time_t)0;
 	sess->ssi.waiting_for_ack = 0;
-
+	sess->icq_info = NULL;
 	sess->authinfo = NULL;
 	sess->emailinfo = NULL;
-
-	/*
-	 * Default to SNAC login unless XORLOGIN is explicitly set.
-	 */
-	if (!(flags & AIM_SESS_FLAGS_XORLOGIN))
-		sess->flags |= AIM_SESS_FLAGS_SNACLOGIN;
-	sess->flags |= flags;
+	sess->oft_info = NULL;
 
 	/*
 	 * This must always be set.  Default to the queue-based
@@ -906,12 +869,11 @@ faim_export void aim_session_init(aim_session_t *sess, fu32_t flags, int debugle
 	 */
 	aim_tx_setenqueue(sess, AIM_TX_QUEUED, NULL);
 
-
 	/*
 	 * Register all the modules for this session...
 	 */
 	aim__registermodule(sess, misc_modfirst); /* load the catch-all first */
-	aim__registermodule(sess, general_modfirst);
+	aim__registermodule(sess, service_modfirst);
 	aim__registermodule(sess, locate_modfirst);
 	aim__registermodule(sess, buddylist_modfirst);
 	aim__registermodule(sess, msg_modfirst);
@@ -925,8 +887,8 @@ faim_export void aim_session_init(aim_session_t *sess, fu32_t flags, int debugle
 	aim__registermodule(sess, translate_modfirst);
 	aim__registermodule(sess, chatnav_modfirst);
 	aim__registermodule(sess, chat_modfirst);
-	aim__registermodule(sess, newsearch_modfirst);
-	aim__registermodule(sess, icon_modfirst);
+	aim__registermodule(sess, odir_modfirst);
+	aim__registermodule(sess, bart_modfirst);
 	/* missing 0x11 - 0x12 */
 	aim__registermodule(sess, ssi_modfirst);
 	/* missing 0x14 */
@@ -939,9 +901,9 @@ faim_export void aim_session_init(aim_session_t *sess, fu32_t flags, int debugle
 }
 
 /**
- * aim_session_kill - Deallocate a session
- * @sess: Session to kill
+ * Logoff and deallocate a session.
  *
+ * @param sess Session to kill
  */
 faim_export void aim_session_kill(aim_session_t *sess)
 {
@@ -955,14 +917,15 @@ faim_export void aim_session_kill(aim_session_t *sess)
 }
 
 /**
- * aim_setdebuggingcb - Set the function to call when outputting debugging info
- * @sess: Session to change
- * @cb: Function to call
+ * Set the function to call when outputting debugging info.
  *
  * The function specified is called whenever faimdprintf() is used within
  * libfaim, and the session's debugging level is greater tha nor equal to
  * the value faimdprintf was called with.
  *
+ * @param sess Session to change.
+ * @param cb Function to call.
+ * @return Returns -1 if the session does not exist, zero otherwise.
  */
 faim_export int aim_setdebuggingcb(aim_session_t *sess, faim_debugging_callback_t cb)
 {
@@ -976,13 +939,12 @@ faim_export int aim_setdebuggingcb(aim_session_t *sess, faim_debugging_callback_
 }
 
 /**
- * aim_conn_isconnecting - Determine if a connection is connecting
- * @conn: Connection to examine
- *
- * Returns nonzero if the connection is in the process of
- * connecting (or if it just completed and aim_conn_completeconnect()
- * has yet to be called on it).
+ * Determine if a connection is connecting.
  *
+ * @param conn Connection to examine.
+ * @return Returns nonzero if the connection is in the process of
+ *         connecting (or if it just completed and
+ *         aim_conn_completeconnect() has yet to be called on it).
  */
 faim_export int aim_conn_isconnecting(aim_conn_t *conn)
 {
@@ -1028,25 +990,22 @@ faim_export aim_session_t *aim_conn_getsess(aim_conn_t *conn)
 	return (aim_session_t *)conn->sessv;
 }
 
-/*
- * aim_logoff()
- *
- * Closes -ALL- open connections.
+/**
+ * Close -ALL- open connections.
  *
+ * @param sess The session.
+ * @return Zero.
  */
 faim_export int aim_logoff(aim_session_t *sess)
 {
-
 	aim_connrst(sess);  /* in case we want to connect again */
 
 	return 0;
 }
 
-/*
- * aim_flap_nop()
- *
- * No-op.  WinAIM 4.x sends these _every minute_ to keep
- * the connection alive.
+/**
+ * No-op.  This sends an empty channel 5 SNAC.  WinAIM 4.x and higher
+ * sends these _every minute_ to keep the connection alive.
  */
 faim_export int aim_flap_nop(aim_session_t *sess, aim_conn_t *conn)
 {
@@ -1057,5 +1016,8 @@ faim_export int aim_flap_nop(aim_session_t *sess, aim_conn_t *conn)
 
 	aim_tx_enqueue(sess, fr);
 
+	/* clean out SNACs over 60sec old */
+	aim_cleansnacs(sess, 60);
+
 	return 0;
 }
