@@ -552,20 +552,34 @@ gaim_str_to_time(const char *timestamp, gboolean utc)
 					tzoff *= -1;
 			}
 
+			t->tm_isdst = -1;
+
 			if (tzoff || utc) {
+#ifdef _WIN32
+				TIME_ZONE_INFORMATION tzi;
+				DWORD ret;
+				if ((ret = GetTimeZoneInformation(&tzi))
+						!= TIME_ZONE_ID_INVALID) {
+					tzoff -= tzi.Bias * 60;
+					if (ret == TIME_ZONE_ID_DAYLIGHT) {
+						tzoff -= tzi.DaylightBias * 60;
+					}
+				}
+#else
 #ifdef HAVE_TM_GMTOFF
 				tzoff += t->tm_gmtoff;
 #else
 #	ifdef HAVE_TIMEZONE
 				tzset();    /* making sure */
 				tzoff -= timezone;
+				t->tm_isdst = 0; /* I think this might fix it */
 #	endif
 #endif
+#endif /* _WIN32 */
 			}
 		}
 	}
 
-	t->tm_isdst = -1;
 	retval = mktime(t);
 	retval += tzoff;
 
@@ -1090,7 +1104,10 @@ gaim_markup_html_to_xhtml(const char *html, char **xhtml_out,
 						}
 						p++;
 					}
-					c = strchr(c, '>') + 1;
+					if ((c = strchr(c, '>')) != NULL)
+						c++;
+					else
+						c = p;
 					pt = g_new0(struct gaim_parse_tag, 1);
 					pt->src_tag = "font";
 					pt->dest_tag = "span";
@@ -1118,7 +1135,10 @@ gaim_markup_html_to_xhtml(const char *html, char **xhtml_out,
 							}
 							g_string_append_printf(xhtml, "<span style='background: %s;'>", color->str);
 							g_string_free(color, TRUE);
-							c = strchr(c, '>') + 1;
+							if ((c = strchr(c, '>')) != NULL)
+								c++;
+							else
+								c = p;
 							pt->src_tag = "body";
 							pt->dest_tag = "span";
 							tags = g_list_prepend(tags, pt);
@@ -1309,7 +1329,7 @@ gaim_markup_strip_html(const char *str)
 					cdata_close_tag = "</style>";
 				}
 				/* Update the index and continue checking after the tag */
-				i = (str2[k] == '<')? k - 1: k;
+				i = (str2[k] == '<' || str2[k] == '\0')? k - 1: k;
 				continue;
 			}
 		}
@@ -1402,8 +1422,7 @@ char *
 gaim_markup_linkify(const char *text)
 {
 	const char *c, *t, *q = NULL;
-	char *tmp, *tmpurlbuf;
-	char url_buf[BUF_LEN * 4];
+	char *tmp, *tmpurlbuf, *url_buf;
 	gunichar g;
 	gboolean inside_html = FALSE;
 	int inside_paren = 0;
@@ -1459,11 +1478,11 @@ gaim_markup_linkify(const char *text)
 						t--;
 					}
 
-					strncpy(url_buf, c, t - c);
-					url_buf[t - c] = 0;
+					url_buf = g_strndup(c, t - c);
 					tmpurlbuf = gaim_unescape_html(url_buf);
 					g_string_append_printf(ret, "<A HREF=\"%s\">%s</A>",
 							tmpurlbuf, url_buf);
+					g_free(url_buf);
 					g_free(tmpurlbuf);
 					c = t;
 					break;
@@ -1492,12 +1511,12 @@ gaim_markup_linkify(const char *text)
 						if ((*(t - 1) == ')' && (inside_paren > 0))) {
 							t--;
 						}
-						strncpy(url_buf, c, t - c);
-						url_buf[t - c] = 0;
+						url_buf = g_strndup(c, t - c);
 						tmpurlbuf = gaim_unescape_html(url_buf);
 						g_string_append_printf(ret,
 								"<A HREF=\"http://%s\">%s</A>", tmpurlbuf,
 								url_buf);
+						g_free(url_buf);
 						g_free(tmpurlbuf);
 						c = t;
 						break;
@@ -1516,11 +1535,11 @@ gaim_markup_linkify(const char *text)
 					if ((*(t - 1) == ')' && (inside_paren > 0))) {
 						t--;
 					}
-					strncpy(url_buf, c, t - c);
-					url_buf[t - c] = 0;
+					url_buf = g_strndup(c, t - c);
 					tmpurlbuf = gaim_unescape_html(url_buf);
 					g_string_append_printf(ret, "<A HREF=\"%s\">%s</A>",
 							tmpurlbuf, url_buf);
+					g_free(url_buf);
 					g_free(tmpurlbuf);
 					c = t;
 					break;
@@ -1543,12 +1562,12 @@ gaim_markup_linkify(const char *text)
 						if ((*(t - 1) == ')' && (inside_paren > 0))) {
 							t--;
 						}
-						strncpy(url_buf, c, t - c);
-						url_buf[t - c] = 0;
+						url_buf = g_strndup(c, t - c);
 						tmpurlbuf = gaim_unescape_html(url_buf);
 						g_string_append_printf(ret,
 								"<A HREF=\"ftp://%s\">%s</A>", tmpurlbuf,
 								url_buf);
+						g_free(url_buf);
 						g_free(tmpurlbuf);
 						c = t;
 						break;
@@ -1564,11 +1583,11 @@ gaim_markup_linkify(const char *text)
 				if (badchar(*t) || badentity(t)) {
 					if (*(t - 1) == '.')
 						t--;
-					strncpy(url_buf, c, t - c);
-					url_buf[t - c] = 0;
+					url_buf = g_strndup(c, t - c);
 					tmpurlbuf = gaim_unescape_html(url_buf);
 					g_string_append_printf(ret, "<A HREF=\"%s\">%s</A>",
 							  tmpurlbuf, url_buf);
+					g_free(url_buf);
 					g_free(tmpurlbuf);
 					c = t;
 					break;
@@ -1580,17 +1599,17 @@ gaim_markup_linkify(const char *text)
 			}
 		} else if (c != text && (*c == '@')) {
 			int flag;
-			GString *gurl_buf;
+			GString *gurl_buf = NULL;
 			const char illegal_chars[] = "!@#$%^&*()[]{}/|\\<>\":;\r\n \0";
-			url_buf[0] = 0;
 
 			if (strchr(illegal_chars,*(c - 1)) || strchr(illegal_chars, *(c + 1)))
 				flag = 0;
-			else
+			else {
 				flag = 1;
+				gurl_buf = g_string_new("");
+			}
 
 			t = c;
-			gurl_buf = g_string_new("");
 			while (flag) {
 				/* iterate backwards grabbing the local part of an email address */
 				g = g_utf8_get_char(t);
@@ -1618,11 +1637,13 @@ gaim_markup_linkify(const char *text)
 				if (badchar(*t) || (g >= 127) || (*t == ')') || badentity(t)) {
 					char *d;
 
-					strcpy(url_buf, gurl_buf->str);
+					url_buf = g_string_free(gurl_buf, FALSE);
 
 					/* strip off trailing periods */
-					for (d = url_buf + strlen(url_buf) - 1; *d == '.'; d--, t--)
-						*d = '\0';
+					if (strlen(url_buf) > 0) {
+						for (d = url_buf + strlen(url_buf) - 1; *d == '.'; d--, t--)
+							*d = '\0';
+					}
 
 					tmpurlbuf = gaim_unescape_html(url_buf);
 					if (gaim_email_is_valid(tmpurlbuf)) {
@@ -1631,6 +1652,7 @@ gaim_markup_linkify(const char *text)
 					} else {
 						g_string_append(ret, url_buf);
 					}
+					g_free(url_buf);
 					g_free(tmpurlbuf);
 					c = t;
 
@@ -1640,7 +1662,6 @@ gaim_markup_linkify(const char *text)
 					t = g_utf8_find_next_char(t, NULL);
 				}
 			}
-			g_string_free(gurl_buf, TRUE);
 		}
 
 		if(*c == ')' && !inside_html) {
@@ -1920,6 +1941,14 @@ int gaim_build_dir (const char *path, int mode)
 
 		if(g_file_test(dir, G_FILE_TEST_IS_DIR)) {
 			continue;
+#ifdef _WIN32
+		/* allow us to create subdirs on UNC paths
+		 * (\\machinename\path\to\blah)
+		 * g_file_test() doesn't work on "\\machinename" */
+		} else if (cur == 2 && dir[0] == '\\' && dir[1] == '\\'
+				&& components[cur + 1] != NULL) {
+			continue;
+#endif
 		} else if(g_file_test(dir, G_FILE_TEST_EXISTS)) {
 			gaim_debug_warning("build_dir", "bad path: %s\n", path);
 			g_strfreev(components);
@@ -1927,7 +1956,7 @@ int gaim_build_dir (const char *path, int mode)
 			return -1;
 		}
 
-		if (mkdir(dir, mode) < 0) {
+		if (g_mkdir(dir, mode) < 0) {
 			gaim_debug_warning("build_dir", "mkdir: %s\n", strerror(errno));
 			g_strfreev(components);
 			g_free(dir);
@@ -1972,7 +2001,7 @@ gaim_mkstemp(char **fpath)
 						   "Problem creating the template\n");
 			else
 			{
-				if( (fp = fopen( result, "w+" )) == NULL ) {
+				if( (fp = g_fopen( result, "w+" )) == NULL ) {
 					gaim_debug(GAIM_DEBUG_ERROR, "gaim_mkstemp",
 							   "Couldn't fopen() %s\n", result);
 				}
@@ -2144,6 +2173,8 @@ gaim_str_sub_away_formatters(const char *str, const char *name)
 				default:
 					cpy[cnt++] = *c;
 				}
+			} else {
+				cpy[cnt++] = *c;
 			}
 			break;
 		default:
@@ -2370,7 +2401,7 @@ gaim_str_size_to_units(size_t size)
 	else {
 		size_mag = (float)size;
 
-		while ((size_index < 4) && (size_mag > 1024)) {
+		while ((size_index < 3) && (size_mag > 1024)) {
 			size_mag /= 1024;
 			size_index++;
 		}
@@ -2569,6 +2600,9 @@ parse_redirect(const char *data, size_t data_len, gint sock,
 		if (end == NULL)
 			end = strchr(s, '\n');
 
+		if (end == NULL)
+			return FALSE;
+
 		len = end - s;
 
 		new_url = g_malloc(len + 1);
@@ -2893,19 +2927,29 @@ gaim_url_decode(const char *str)
 const char *
 gaim_url_encode(const char *str)
 {
+	const char *iter;
 	static char buf[BUF_LEN];
+	char utf_char[6];
 	guint i, j = 0;
 
 	g_return_val_if_fail(str != NULL, NULL);
+	g_return_val_if_fail(g_utf8_validate(str, -1, NULL), NULL);
 
-	for (i = 0; i < strlen(str) && j < (BUF_LEN - 1); i++) {
-		if (isalnum(str[i]))
-			buf[j++] = str[i];
-		else {
-			if (j > (BUF_LEN - 4))
-				break;
-			sprintf(buf + j, "%%%02x", (unsigned char)str[i]);
-			j += 3;
+	iter = str;
+	for (; *iter && j < (BUF_LEN - 1) ; iter = g_utf8_next_char(iter)) {
+		gunichar c = g_utf8_get_char(iter);
+		/* If the character is an ASCII character and is alphanumeric
+		 * no need to escape */
+		if (c < 128 && isalnum(c)) {
+			buf[j++] = c;
+		} else {
+			int bytes = g_unichar_to_utf8(c, utf_char);
+			for (i = 0; i < bytes; i++) {
+				if (j > (BUF_LEN - 4))
+					break;
+				sprintf(buf + j, "%%%02x", utf_char[i] & 0xff);
+				j += 3;
+			}
 		}
 	}
 
@@ -3258,20 +3302,30 @@ const char* gaim_unescape_filename(const char *escaped) {
 const char *
 gaim_escape_filename(const char *str)
 {
+	const char *iter;
 	static char buf[BUF_LEN];
+	char utf_char[6];
 	guint i, j = 0;
 
 	g_return_val_if_fail(str != NULL, NULL);
+	g_return_val_if_fail(g_utf8_validate(str, -1, NULL), NULL);
 
-	for (i = 0; i < strlen(str) && j < (BUF_LEN - 1); i++) {
-		if (isalnum(str[i]) || str[i] == '@' || str[i] == '-' ||
-				str[i] == '_' || str[i] == '.' || str[i] == '#')
-			buf[j++] = str[i];
-		else {
-			if (j > (BUF_LEN - 4))
-				break;
-			sprintf(buf + j, "%%%02x", (unsigned char)str[i]);
-			j += 3;
+	iter = str;
+	for (; *iter && j < (BUF_LEN - 1) ; iter = g_utf8_next_char(iter)) {
+		gunichar c = g_utf8_get_char(iter);
+		/* If the character is an ASCII character and is alphanumeric,
+		 * or one of the specified values, no need to escape */
+		if (c < 128 && (isalnum(c) || c == '@' || c == '-' ||
+				c == '_' || c == '.' || c == '#')) {
+			buf[j++] = c;
+		} else {
+			int bytes = g_unichar_to_utf8(c, utf_char);
+			for (i = 0; i < bytes; i++) {
+				if (j > (BUF_LEN - 4))
+					break;
+				sprintf(buf + j, "%%%02x", utf_char[i] & 0xff);
+				j += 3;
+			}
 		}
 	}
 
